# Original authors: Andres Torres and Maximiliano Cecilia

import os.path
import itertools
try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO
from dropbox import Dropbox
from dropbox.exceptions import ApiError
from dropbox.files import FolderMetadata, FileMetadata, DeleteError
from django.core.exceptions import ImproperlyConfigured
from django.core.files import File
from django.core.files.storage import Storage
from django.utils.deconstruct import deconstructible

from .settings import ACCESS_TOKEN, ROOT_FOLDER, DEFAULT_ROOT_FOLDER


@deconstructible
class DropboxStorage(Storage):
    """
    A storage class providing access to resources in a Dropbox folder.
    """

    def __init__(self, token=ACCESS_TOKEN, location=ROOT_FOLDER):
        if not token:
            raise ImproperlyConfigured("You must configure an access token at "
                                       "'settings.DROPBOX_ACCESS_TOKEN'.")

        self.client = Dropbox(token)
        self.account_info = self.client.users_get_current_account()
        self.location = location or DEFAULT_ROOT_FOLDER
        self.base_url = 'https://dl.dropboxusercontent.com/'

    def _get_abs_path(self, name):
        return os.path.realpath(os.path.join(self.location, name))

    def _open(self, name, mode='rb'):
        name = self._get_abs_path(name)
        remote_file = DropboxFile(name, self, mode=mode)
        return remote_file

    def _save(self, name, content):
        name = self._get_abs_path(name)
        directory = os.path.dirname(name)
        if not self.exists(directory) and directory:
            self.client.files_create_folder(directory)
        # response = self.client.files_get_metadata(directory)
        # if not response['is_dir']:
        #     raise IOError("%s exists and is not a directory." % directory)
        abs_name = os.path.realpath(os.path.join(self.location, name))
        self.client.files_upload(content.read(), abs_name)
        return name

    def delete(self, name):
        name = self._get_abs_path(name)
        try:
            self.client.files_delete(name)
        except ApiError as e:
            if isinstance(e.error, DeleteError)\
                    and e.error.is_path_lookup()\
                    and e.error.get_path_lookup().is_not_found():
                # not found
                return False
            # error
            raise e
        # deleted
        return True

    def exists(self, name):
        name = self._get_abs_path(name)
        try:
            self.client.files_get_metadata(name)
        except ApiError as e:
            if hasattr(e.error, 'is_path')\
                    and e.error.is_path()\
                    and e.error.get_path().is_not_found():
                # not found
                return False
            # error
            raise e
        # found
        return True

    def listdir(self, path):
        path = self._get_abs_path(path)
        response = self.client.files_list_folder(path)
        directories = []
        files = []
        for entry in response.entries:
            if isinstance(entry, FolderMetadata):
                directories.append(os.path.basename(entry.path_display))
            elif isinstance(entry, FileMetadata):
                files.append(os.path.basename(entry.path_display))
        return directories, files

    def size(self, name):
        name = self._get_abs_path(name)
        return self.client.files_get_metadata(name).size

    def url(self, name):
        name = self._get_abs_path(name)
        return self.client.files_get_temporary_link(name).link

    def modified_time(self, name):
        name = self._get_abs_path(name)
        return self.client.files_get_metadata(name).server_modified

    def accessed_time(self, name):
        name = self._get_abs_path(name)
        # Note to the unwary, this is actually an mtime
        return self.client.files_get_metadata(name).client_modified

    def get_available_name(self, name, max_length=None):
        """
        Returns a filename that's free on the target storage system, and
        available for new content to be written to.
        """
        name = self._get_abs_path(name)
        dir_name, file_name = os.path.split(name)
        file_root, file_ext = os.path.splitext(file_name)
        # If the filename already exists, add an underscore and a number (before
        # the file extension, if one exists) to the filename until the generated
        # filename doesn't exist.
        count = itertools.count(1)
        while self.exists(name):
            # file_ext includes the dot.
            _fn = "%s_%s%s" % (file_root, count.next(), file_ext)
            name = os.path.join(dir_name, _fn)

        return name


class DropboxFile(File):
    def __init__(self, name, storage, mode):
        self._storage = storage
        self._mode = mode
        self._is_dirty = False
        self.file = StringIO()
        self.start_range = 0
        self._name = name

    @property
    def size(self):
        if not hasattr(self, '_size'):
            self._size = self._storage.size(self._name)
        return self._size

    def read(self, num_bytes=None):
        metadata, response = self._storage.client.files_download(self._name)
        return response.content

    def write(self, content):
        if 'w' not in self._mode:
            raise AttributeError("File was opened for read-only access.")
        self.file = StringIO(content)
        self._is_dirty = True

    def close(self):
        if self._is_dirty:
            self._storage.client.files_upload(self.file.getvalue(), self._name)
        self.file.close()



# Original authors: Andres Torres and Maximiliano Cecilia

from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import BaseCommand, CommandError
from dropbox import DropboxOAuth2FlowNoRedirect

from django_dropbox_storage.settings import CONSUMER_KEY, CONSUMER_SECRET


class Command(BaseCommand):

    def handle(self, *args, **options):
        if not (CONSUMER_KEY and CONSUMER_SECRET):
            raise ImproperlyConfigured("To use this tool you have to set "
                                       "'settings.DROPBOX_CONSUMER_KEY' and "
                                       "'settings.DROPBOX_CONSUMER_SECRET'.")

        auth_flow = DropboxOAuth2FlowNoRedirect(CONSUMER_KEY, CONSUMER_SECRET)

        authorize_url = auth_flow.start()
        self.stdout.write('1. Go to: {}'.format(authorize_url))
        self.stdout.write('2. Click "Allow" (you might have to log in first).')
        self.stdout.write('3. Copy the authorization code.')
        auth_code = input("Enter the authorization code here: ").strip()

        try:
            oauth_result = auth_flow.finish(auth_code)
            token = oauth_result.access_token
            self.stdout.write("DROPBOX_ACCESS_TOKEN = '{}'".format(token))
        except Exception as e:
            raise CommandError('Error: {}'.format(e))



from django.conf import settings

DEFAULT_ROOT_FOLDER = '/'

CONSUMER_KEY = getattr(settings, 'DROPBOX_CONSUMER_KEY', None)

CONSUMER_SECRET = getattr(settings, 'DROPBOX_CONSUMER_SECRET', None)

ACCESS_TOKEN = getattr(settings, 'DROPBOX_ACCESS_TOKEN', None)

ROOT_FOLDER = getattr(settings, 'DROPBOX_ROOT_FOLDER', DEFAULT_ROOT_FOLDER)



#!/usr/bin/env python
from os import path
from codecs import open
from setuptools import setup, find_packages
from django_dropbox_storage import version

packages = find_packages(exclude=['contrib', 'docs', 'tests'])

requires = [
    'django>=3.1.4',
    'dropbox>=11.0.0',
]

project_url = 'https://github.com/ryesalvador/django-dropbox-storage'

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-dropbox-storage-fork',
    version=version,
    description='A Dropbox Storage for your Django apps',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=u'Emanuele Bertoldi',
    author_email='emanuele.bertoldi@gmail.com',
    url=project_url,
    download_url=project_url + '/archive/' + version + '.tar.gz',
    packages=packages,
    install_requires=requires,
    keywords = 'django storage dropbox',
    license='MIT',
)
